@@ -35,19 +35,23 @@
     dbname
 }
 
-.fix_numeric_cols <- function(df)
-{
-    col_idx <- which(sapply(df, is.numeric))
-    df[col_idx] <- lapply(df[col_idx], as.integer)
-    df
+.restrict_to_tx_attrib <- function(SQL, tx_attrib) {
+    paste0(SQL,
+           " JOIN transcript_attrib AS ta",
+           " USING(transcript_id)",
+           " WHERE ta.attrib_type_id=",
+           "(SELECT attrib_type_id FROM attrib_type WHERE code='", tx_attrib,
+           "')")
 }
 
-.RMySQL_select <- function(dbconn, columns, from)
+.dbselect <- function(dbconn, columns, from, tx_attrib = NULL)
 {
-    SQL <- sprintf("SELECT %s FROM %s", paste0(columns, collapse=","), from)
-    ## Not sure systematic conversion of numeric to int is actually a
-    ## good idea (risk of overflow?)
-    .fix_numeric_cols(suppressWarnings(dbGetQuery(dbconn, SQL)))
+    SQL <- sprintf("SELECT %s FROM %s", paste0(columns, collapse=","),
+                   from)
+    if (!is.null(tx_attrib)) {
+        SQL <- .restrict_to_tx_attrib(SQL, tx_attrib)
+    }
+    dbGetQuery(dbconn, SQL)
 }
 
 .seq_region_columns <- c(
@@ -57,7 +61,7 @@
     "seq_region_strand"
 )
 
-.fetch_Ensembl_transcripts <- function(dbconn)
+.fetch_Ensembl_transcripts <- function(dbconn, tx_attrib)
 {
     message("Fetch transcripts and genes from Ensembl ... ",
             appendLF=FALSE)
@@ -69,7 +73,7 @@
     )
     columns <- c(paste0("transcript.", transcript_columns), "gene.stable_id")
     from <- "transcript LEFT JOIN gene USING(gene_id)"
-    transcripts <- .RMySQL_select(dbconn, columns, from)
+    transcripts <- .dbselect(dbconn, columns, from, tx_attrib)
     colnames(transcripts) <- c("tx_id",
                                "tx_name",
                                "seq_region_id",
@@ -83,7 +87,7 @@
     transcripts
 }
 
-.fetch_Ensembl_translations <- function(dbconn)
+.fetch_Ensembl_translations <- function(dbconn, tx_attrib)
 {
     columns <- c(
         "stable_id",
@@ -93,7 +97,7 @@
         "seq_end",         # relative to last exon
         "transcript_id"
     )
-    .RMySQL_select(dbconn, columns, "translation")
+    .dbselect(dbconn, columns, "translation", tx_attrib)
 }
 
 ### 'has_cds' must be a logical vector.
@@ -115,9 +119,9 @@
 }
 
 ### Add "cds_name", "cds_start", and "cds_end" cols to 'splicings'.
-.add_cds_cols <- function(dbconn, splicings)
+.add_cds_cols <- function(dbconn, splicings, tx_attrib)
 {
-    translations <- .fetch_Ensembl_translations(dbconn)
+    translations <- .fetch_Ensembl_translations(dbconn, tx_attrib)
 
     m <- match(splicings$tx_id, translations$transcript_id)
     cds_name <- translations$stable_id[m]
@@ -150,7 +154,7 @@
     cbind(splicings, cds_name, cds_start, cds_end, stringsAsFactors=FALSE)
 }
 
-.fetch_Ensembl_splicings <- function(dbconn)
+.fetch_Ensembl_splicings <- function(dbconn, tx_attrib)
 {
     message("Fetch exons and CDS from Ensembl ... ",
             appendLF=FALSE)
@@ -161,7 +165,7 @@
     )
     columns <- c("transcript_id", "rank", exon_columns)
     from <- "exon_transcript INNER JOIN exon USING(exon_id)"
-    splicings <- .RMySQL_select(dbconn, columns, from)
+    splicings <- .dbselect(dbconn, columns, from, tx_attrib)
     colnames(splicings) <- c("tx_id",
                              "exon_rank",
                              "exon_id",
@@ -173,7 +177,7 @@
     splicings$exon_strand <- strand(splicings$exon_strand)
     oo <- S4Vectors:::orderIntegerPairs(splicings$tx_id, splicings$exon_rank)
     splicings <- S4Vectors:::extract_data_frame_rows(splicings, oo)
-    splicings <- .add_cds_cols(dbconn, splicings)
+    splicings <- .add_cds_cols(dbconn, splicings, tx_attrib)
     message("OK")
     splicings
 }
@@ -185,7 +189,7 @@
     ## extract this value from Ensembl
     id0 <- 6L
     columns <- c("seq_region_id", "attrib_type_id", "value")
-    seq_region_attrib <- .RMySQL_select(dbconn, columns, "seq_region_attrib")
+    seq_region_attrib <- .dbselect(dbconn, columns, "seq_region_attrib")
     seq_region_attrib$seq_region_id[seq_region_attrib$attrib_type_id == id0]
 }
 
@@ -213,7 +217,7 @@
                         setdiff(seq_region_columns, using_column),
                         setdiff(coord_system_columns, using_column))
     from <- "seq_region INNER JOIN coord_system USING(coord_system_id)"
-    seq_region <- .RMySQL_select(dbconn, "*", from)
+    seq_region <- .dbselect(dbconn, "*", from)
     stopifnot(identical(colnames(seq_region), joined_columns))
     colnames(seq_region)[6:9] <- paste0("coord_system_",
                                         colnames(seq_region)[6:9])
@@ -257,17 +261,24 @@
 makeTxDbFromEnsembl <- function(organism="Homo sapiens",
                                 release=NA,
                                 circ_seqs=DEFAULT_CIRC_SEQS,
-                                server="ensembldb.ensembl.org")
+                                server="ensembldb.ensembl.org",
+                                username="anonymous", password=NULL,
+                                port=0L, tx_attrib=NULL)
 {
+    if (!requireNamespace("RMariaDB", quietly=TRUE))
+        stop(wmsg("Couldn't load the RMariaDB package. ",
+                  "You need to install the RMariaDB package ",
+                  "in order to use makeTxDbFromEnsembl()."))
+
     dbname <- .lookup_dbname(organism, release=release)
-    dbconn <- dbConnect(MySQL(), dbname=dbname,
-                                 username="anonymous",
-                                 host=server)
+    dbconn <- dbConnect(RMariaDB::MariaDB(), dbname=dbname,
+                        host=server, username=username, password=password,
+                        port=port)
     on.exit(dbDisconnect(dbconn))
 
-    transcripts <- .fetch_Ensembl_transcripts(dbconn)
+    transcripts <- .fetch_Ensembl_transcripts(dbconn, tx_attrib)
 
-    splicings <- .fetch_Ensembl_splicings(dbconn)
+    splicings <- .fetch_Ensembl_splicings(dbconn, tx_attrib)
 
     seq_region_ids <- unique(c(transcripts$seq_region_id,
                                splicings$seq_region_id))
